local type = type
local pairs = pairs
local pcall = pcall
local xpcall = xpcall
local tostring = tostring
local str_substr = string.sub
local str_replace = string.gsub
local str_format = string.format
local str_repeat = string.rep
local str_ord = string.byte
local str_chr = string.char
local table_size = table_size
local arr_insert = table.insert
local arr_remove = table.remove
local arr_sort = table.sort
local arr_concat = table.concat
local arr_unpack = table.unpack
local arr_empty = {}
local tbl_count
do
local tbl_count_size_classes = {
["LuaCustomTable"] = true,
["LuaTransportLine"] = true,
["LuaFluidBox"] = true,
["LuaInventory"] = true,
}
tbl_count = function(tbl)
if type(tbl) == "userdata" and tbl_count_size_classes[tbl.object_name] then
return #tbl
end
return table_size(tbl)
end
end
local function tbl_select(tbl,...)
local arr = {}
for k,v in pairs{...} do
arr[k] = tbl[v]
end
return arr
end
local function tbl_getter_func(tbl)
return function(key)
return tbl[key]
end
end
local function tbl_get_raw(tbl,key)
return tbl[key]
end
local tbl_vector_to_array_lookup = {1,2,3,4,x=1,y=2,z=3,w=4}
local function tbl_vector_to_array(tbl)
local ret = {}
for k,v in pairs(tbl) do
ret[tbl_vector_to_array_lookup[k]] = v
end
return ret
end
local function default_value(val,default)
if type(val) == type(default) then return val end
return default
end
local function tbl_set_default(tbl,key,default)
local ret = tbl[key]
if type(ret) == type(default) then return ret end
tbl[key] = default
return default
end
local function tbl_get_tbl(tbl,key)
local ret = tbl[key]
if type(ret) ~= "table" then
ret = {}
tbl[key] = ret
end
return ret
end
local function tbl_set_deep(tbl,...)
local t = tbl
local keys = {...}
local val = arr_remove(keys)
local key = arr_remove(keys)
for _,k in pairs(keys) do
t = tbl_get_tbl(t,k)
end
t[key] = val
end
local function is_not_table(obj)
local t = type(obj)
return t ~= "table" and (t ~= "userdata" or obj.object_name ~= "LuaCustomTable")
end
local function tbl_copy(...)
local lookup = {}
local function tcp(tbl)
if is_not_table(tbl) then return tbl end
if lookup[tbl] then return lookup[tbl] end
local ret = {}
lookup[tbl] = ret
for k,v in pairs(tbl) do
ret[tcp(k)] = tcp(v)
end
lookup[tbl] = nil
return ret
end
return tcp(...)
end
local function tbl_clone(...)
local lookup = {}
local function tcp(tbl)
if is_not_table(tbl) then return tbl end
if lookup[tbl] then return lookup[tbl] end
local ret = {}
lookup[tbl] = ret
for k,v in pairs(tbl) do
ret[tcp(k)] = tcp(v)
end
return ret
end
return tcp(...)
end
local function tbl_eq(...)
local lookup = {}
local function teq(a,b)
if a == b then return true end -- short circuit on the same table/value
if is_not_table(a) or is_not_table(b) then return false end -- only if both are tables they can be equal
if lookup[a] and lookup[a][b] or lookup[b] and lookup[b][a] then return true end -- recursive tables already under investigation are equal until proven otherwise
if lookup[a] then -- keep lookup small
lookup[a][b] = true
elseif lookup[b] then -- keep lookup small
lookup[b][a] = true
else
lookup[a] = {[b]=true}
end
for k,v in pairs(a) do
if not teq(b[k],v) then
return false
end
end
for k,v in pairs(b) do
if a[k] == nil and v ~= nil then
return false
end
end
return true
end
return teq(...)
end
local function arr_add_new(arr,val)
for _,v in pairs(arr) do
if tbl_eq(v,val) then return v end
end
arr_insert(arr,val)
return val
end
local function arr_unique(arr)
for a = #arr-1,1,-1 do -- start at the end for faster array shortening
for b = #arr,a+1,-1 do
if tbl_eq(arr[a],arr[b]) then
arr_remove(arr,b)
end
end
end
end
local function tbl_keys(tbl,insert_func,sort_func)
if insert_func == nil then insert_func = arr_insert end
local arr = {}
for k in pairs(tbl) do
insert_func(arr,k)
end
if sort_func then
arr_sort(arr,sort_func)
end
return arr
end
local function tbl_values(tbl,insert_func,sort_func)
if insert_func == nil then insert_func = arr_insert end
local arr = {}
for _,v in pairs(tbl) do
insert_func(arr,v)
end
if sort_func then
arr_sort(arr,sort_func)
end
return arr
end
local function tbl_get(tbl,key,default)
for k,v in pairs(tbl) do
if tbl_eq(k,key) then
return v
end
end
return default
end
local function tbl_find_value(tbl,val)
for k,v in pairs(tbl) do
if tbl_eq(v,val) then
return k,v
end
end
end
local function tbl_flip(tbl)
local r = {}
for k,v in pairs(tbl) do
r[v] = k
end
return r
end
local function tbl_flip_multi(tbl)
local r = {}
for k,v in pairs(tbl) do
arr_add_new(
tbl_get_tbl(r,
arr_add_new(tbl_keys(r,arr_add_new),v)
)
,k)
end
return r
end
local function get_prototype_name(obj)
if obj.object_name == "LuaRecord" then return obj.type end
return obj.name
end
local function get_prototype_type(obj)
if obj.object_name == "LuaRecord" then return obj.type end
return obj.type
end
local get_ntt_prototype
local get_item_prototype
local get_quality_prototype
local function get_prototype(...) -- use actual prototype basetype strings
local prototypes = prototypes
local pt = {}
for k in pairs(defines.prototypes) do
local err,val = pcall(tbl_get_raw,prototypes,str_replace(k,"-","_"))
if err then
pt[k] = val
end
end
get_prototype = function(typ,name) -- typ uses prototype type strings
local tbl = pt[typ]
return tbl and tbl[name]
end
get_ntt_prototype = tbl_getter_func(pt["entity"])
get_item_prototype = tbl_getter_func(pt["item"])
get_quality_prototype = tbl_getter_func(pt["quality"])
return get_prototype(...)
end
get_ntt_prototype = function(name)
return get_prototype("entity",name)
end
get_item_prototype = function(name)
return get_prototype("item",name)
end
get_quality_prototype = function(name)
return get_prototype("quality",name)
end
local function get_prototype_stable(...) -- returned prototypes can be tested for equality
local db = {}
get_prototype_stable = function(typ,name)
local r = db[typ] and db[typ][name]
if r == nil then
r	= get_prototype(typ,name)
if r then
if db[typ] then
db[typ][name] = r
else
db[typ] = { [name]=r }
end
end
end
return r
end
return get_prototype_stable(...)
end
local function get_db(...)
local db = storage
if type(db) ~= "table" then
db = {}
storage = db
end
get_db = function(name)
return tbl_get_tbl(db,name)
end
return get_db(...)
end
local function get_player(...)
get_player = game.get_player
return get_player(...)
end
local function iter_inventory_slots(inv,valid_for_read)
if not inv then return end
local slot = 1
local max = #inv
if valid_for_read==nil then
return function()
for n = slot,max,1 do
local i = inv[n]
if i then
slot = n+1
return i,n
end
end
end
else
return function()
for n = slot,max,1 do
local i = inv[n]
if i and i.valid_for_read == valid_for_read then
slot = n+1
return i,n
end
end
end
end
end
local function iter_inventory(inv,slot)
dump{
inv = inv==nil and "nil" or inv,
slot = slot==nil and "nil" or slot,
}
for n = slot+1,#inv,1 do
local i = inv[n]
if i and i.valid_for_read then
return n,i
end
end
end
local function get_inventories(ntt)
local ret = {}
local get = ntt.get_inventory
for n = 1,ntt.get_max_inventory_index() do
local i = get(n)
if i and #i>0 then
ret[n] = i
end
end
return ret
end
local function get_first_inventory(ntt)
local get = ntt.get_inventory
for n = 1,ntt.get_max_inventory_index() do
local i = get(n)
if i and #i>0 then
return i
end
end
end
local function get_inventory(ntt,num)
if not num then return get_first_inventory(ntt) end
local i = ntt.get_inventory(num)
return i and #i>0 and i
end
local function get_logistic_sections(ntt)
local logi = ntt.get_logistic_sections()
return logi and logi.sections or arr_empty
end
local has_sortable_logistic_sections
do
local has_sortable_logistic_sections_forbidden = defines.logistic_section_type.transitional_request_controlled
has_sortable_logistic_sections = function(ntt)
local logi = ntt.get_logistic_sections()
if not logi then return false end
logi = logi.sections
if not logi then return false end
for _,sect in pairs(logi) do
if sect.type == has_sortable_logistic_sections_forbidden then return false end
end
return true
end
end
local function get_player_settings_value(...)
local get = settings.get_player_settings
get_player_settings_value = function(plr_id,name,default)
local var = get(plr_id)[name]
return var and var.value or default
end
return get_player_settings_value(...)
end
local function dumps(...)
local function dumpt(t)
return tostring(t)
end
local mapped_id,printed_id,next_id
local str_repl = {}
do
for i=0,0x1f do
str_repl[str_chr(i)] = str_format("\\x%02x",i)
end
str_repl["\r"] = "\\r"
str_repl["\n"] = "\\n"
str_repl["\t"] = "\\t"
str_repl["\\"] = "\\\\"
str_repl["\""] = "\\\""
end
local formater = {
["nil"] = function()
return "nil"
end,
["number"] = function(v)
return tostring(v)
end,
["string"] = function(v)
return "\""..str_replace(v,"[\x00-\x1f\n\"]",str_repl).."\""
end,
["boolean"] = function(v)
return v and "true" or "false"
end,
["function"] = dumpt,
["thread"] = dumpt,
}
local function recurse(t,n,i)
local tid = mapped_id[t] or next_id
if tid >= next_id then
mapped_id[t] = tid
next_id = tid +1
end
if not i or printed_id[t] then return n.."#"..tid end
printed_id[t] = true
local ret = {n.."#"..tid}
for k,v in pairs(t) do
arr_insert(ret,arr_concat{
str_repeat("\t",i),
formater[type(k)](k),
" => ",
formater[type(v)](v,i+1),
})
end
return arr_concat(ret,"\n")
end
formater["userdata"] = function(t,i)
if t.object_name ~= "LuaCustomTable" then return dumpt(t) end
return recurse(t,"LuaCustomTable",i)
end
formater["table"] = function(t,i)
return recurse(t,"table",i)
end
dumps = function(...)
mapped_id = {}
printed_id = {}
next_id = 0
local ret = {}
for _,val in pairs{...} do
arr_insert(ret,formater[type(val)](val,1))
end
return arr_concat(ret,"\n")
end
return dumps(...)
end
local function dump(...)
local game = game
local print_state = {skip=defines.print_skip.never,game_state=false}
local function print(txt)
log(txt)
for _,plr in pairs(game.connected_players) do
plr.print(txt,print_state)
end
end
dump = function(...)
return print(dumps(...))
end
return dump(...)
end
local function safecall_err_handler(...)
return arr_concat({
"safecall error:",
arr_concat({...},"\n"),
debug.traceback()
},"\n")
end
local function safecall(func,...)
local ret = {xpcall(func,safecall_err_handler,...)}
local suc = arr_remove(ret,1)
if suc then return arr_unpack(ret) end
log(arr_concat(ret,"\n"))
for _,plr in pairs(game.connected_players) do
plr.print{script.mod_name..".safecall-error",{"mod-name."..script.mod_name},script.mod_name}
end
end
local function safecall_wrapper(func)
return function(...)
return safecall(func,...)
end
end
local function cmp_bp_type(...)
local BlueprintOrder = tbl_flip{"blueprint-book","blueprint","deconstruction-planner","upgrade-planner"}
cmp_bp_type = function(a,b)
local x = BlueprintOrder[a] or 0
local y = BlueprintOrder[b] or 0
if x > y then return false end
if x < y then return true end
return nil
end
return cmp_bp_type(...)
end
local function cmp_str(a,b)
local x = a or ""
local y = b or ""
if x > y then return false end
if x < y then return true end
return nil
end
local function cmp_comparator(...)
local ComparatorOrder = {}
do
for k,v in pairs{{"<"},{"<=","≤"},{">=","≥"},{">"},{"!=","≠"},{"="}} do
for _,c in pairs(v) do
ComparatorOrder[c] = k
end
end
end
cmp_comparator = function(a,b)
local x = ComparatorOrder[a] or 0
local y = ComparatorOrder[b] or 0
if x > y then return false end
if x < y then return true end
return nil
end
return cmp_comparator(...)
end
local function cmp_nil_check(func,a,b)
if a then
if b then
return func(a,b)
end
return false
end
return b and true or nil
end
local function cmp_proto_type(x,y)
if x.order > y.order then return false end
if x.order < y.order then return true end
if x.name > y.name then return false end
if x.name < y.name then return true end
if x.type > y.type then return false end
if x.type < y.type then return true end
return nil
end
local function cmp_proto(x,y)
local c = cmp_proto_type( x.group , y.group )
if c ~= nil then return c end
c = cmp_proto_type( x.subgroup , y.subgroup )
if c ~= nil then return c end
return cmp_proto_type( x , y )
end
local function cmp_qual(x,y)
if x.level > y.level then return false end
if x.level < y.level then return true end
return cmp_proto(x,y)
end
local function cmp_quality(x,y)
return cmp_nil_check(cmp_qual,x,y)
end
local function cmp_prototype(x,y)
return cmp_nil_check(cmp_proto,x,y)
end
local function cmp_prototype_str(...)
local lookup = {}
do -- nothing in here shall generate upvalues
local protos = {}
local order = {}
for n,p in pairs{
["asteroid-chunk"]="asteroid_chunk",
["entity"]="entity",
["equipment"]="equipment",
["fluid"]="fluid",
["item"]="item",
["quality"]="quality",
["recipe"]="recipe",
["space-connection"]="space_connection",
["space-location"]="space_location",
["technology"]="technology",
["tile"]="tile",
["virtual"]="virtual_signal",
} do
lookup[n] = {}
for k,v in pairs(prototypes[p]) do
arr_insert(order,v)
protos[v] = {n,k}
end
end
arr_sort(order,cmp_proto)
for k,v in pairs(order) do
local t,n = arr_unpack(protos[v])
lookup[t][n] = k
end
end -- nothing in here shall generate upvalues
cmp_prototype_str = function(aType,aName,bType,bName)
if aName==bName and aType==bType then return nil end
if not (aType and aName) then
return bType and bName and true or nil
end
if not (bType and bName) then
return false
end
local x,y = lookup[aType][aName],lookup[bType][bName]
if x then
if y then
return x < y
end
return false
end
return y and true or nil
end
return cmp_prototype_str(...)
end
local function cmp_quality_str(...)
local lookup = {}
do -- nothing in here shall generate upvalues
local order = {}
for _,v in pairs(prototypes["quality"]) do
arr_insert(order,v)
end
arr_sort(order,cmp_qual)
for k,v in pairs(order) do
lookup[v.name] = k
end
end -- nothing in here shall generate upvalues
cmp_quality_str = function(a,b)
local x,y = lookup[a],lookup[b]
if x then
if y then
return x < y
end
return false
end
return y and true or nil
end
return cmp_quality_str(...)
end
local function cmp_bp_icons(a,b)
local x = {}
local y = {}
for _,v in pairs(a) do
x[v.index] = v.signal
end
for _,v in pairs(b) do
y[v.index] = v.signal
end
for k = 1,4 do
local c = cmp_prototype_str(
x[k].type or "item",x[k].name,
y[k].type or "item",y[k].name)
if c ~= nil then return c end
c = cmp_quality_str(x[k].quality,y[k].quality)
if c ~= nil then return c end
end
end
local function sort_logistics(a,b)
local x = a.value
local y = b.value
local c = cmp_prototype_str( x.type,x.name , y.type,y.name )
if c ~= nil then return c end
c = cmp_comparator(x.comparator,y.comparator)
if c ~= nil then return c end
return cmp_quality_str(x.quality,y.quality)
end
local function sort_inf_filter(a,b)
local c = cmp_prototype_str( "item",a.name , "item",b.name )
if c ~= nil then return c end
c = cmp_str(a.mode,b.mode)
if c ~= nil then return c end
return cmp_quality_str(a.quality,b.quality)
end
local function sort_decon_filter(a,b)
local c = cmp_prototype_str( "entity",a.name , "entity",b.name )
if c ~= nil then return c end
c = cmp_comparator(a.comparator,b.comparator)
if c ~= nil then return c end
return cmp_quality_str(a.quality,b.quality)
end
local function sort_decon_tile_filter(a,b)
return cmp_prototype_str( "tile",a , "tile",b )
end
local function sort_upgrade_filter(a,b)
local c = cmp_prototype_str( a.type,a.name , b.type,b.name )
if c ~= nil then return c end
c = cmp_comparator(a.comparator,b.comparator)
if c ~= nil then return c end
return cmp_quality_str(a.quality,b.quality)
end
local function sort_book_records(a,b)
local c = cmp_bp_type(a.type,b.type)
if c ~= nil then return c end
return cmp_bp_icons(a.preview_icons,b.preview_icons)
end
local cfg_has_infinity_filter = {
["infinity-cargo-wagon"]=true,
["infinity-container"]=true,
}
local cfg_inventories = {
["asteroid-collector"]={defines.inventory.chest},
["car"]={defines.inventory.car_trunk,defines.inventory.car_trash},
["cargo-landing-pad"]={defines.inventory.cargo_landing_pad_main,defines.inventory.cargo_landing_pad_trash},
["cargo-wagon"]={defines.inventory.cargo_wagon},
["character"]={defines.inventory.character_main,defines.inventory.character_trash},
["self"]={defines.inventory.character_main,defines.inventory.character_trash},
["container"]={defines.inventory.chest},
["infinity-cargo-wagon"]={defines.inventory.cargo_wagon},
["infinity-container"]={defines.inventory.chest},
["linked-container"]={defines.inventory.chest},
["logistic-container"]={defines.inventory.chest,defines.inventory.logistic_container_trash},
["proxy-container"]={defines.inventory.proxy_main},
["roboport"]={defines.inventory.roboport_robot,defines.inventory.roboport_material},
["rocket-silo"]={defines.inventory.rocket_silo_rocket,defines.inventory.rocket_silo_trash},
["space-platform-hub"]={defines.inventory.hub_main,defines.inventory.hub_trash},
["spider-vehicle"]={defines.inventory.spider_trunk,defines.inventory.spider_trash},
}
local cfg_type_map = {
["asteroid-collector"]="chest",
["blueprint-book"]="book",
["car"]="car",
["cargo-landing-pad"]="space",
["cargo-wagon"]="train",
["character"]="player",
["constant-combinator"]="math",
["container"]="chest",
["deconstruction-planner"]="decon",
["infinity-cargo-wagon"]="train",
["infinity-container"]="chest",
["linked-container"]="chest",
["logistic-container"]="chest",
["proxy-container"]="chest",
["roboport"]="robo",
["rocket-silo"]="rocket",
["space-platform-hub"]="space",
["spider-vehicle"]="car",
["upgrade-planner"]="upgrade",
}
local cfg_special_type = {
["player"]=true,
["self"]=true,
}
local data_storage = {}
local function handle_entity_inventory(ntt,num)
local inv = cfg_inventories[ntt.type]
if not inv then return end
inv = inv[num]
if not inv then return end
inv = ntt.get_inventory(inv)
if not inv then return end
inv.sort_and_merge()
end
local function handle_entity_inventory_all(ntt)
local get = ntt.get_inventory
for n = 1,ntt.get_max_inventory_index() do
local i = get(n)
if i and #i>0 then
i.sort_and_merge()
end
end
end
local function handle_entity_logistic_sections(ntt)
for _,sect in pairs(get_logistic_sections(ntt)) do
if sect.is_manual and sect.filters then
local lst = {}
for _,fltr in pairs(sect.filters) do
if fltr.value then
arr_add_new(lst,fltr)
end
end
arr_sort(lst,sort_logistics)
sect.filters = lst
end
end
end
local function handle_entity_infinity_filters(ntt)
if ntt.infinity_container_filters and tbl_count(ntt.infinity_container_filters) > 1 then
local lst = {}
for _,fltr in pairs(ntt.infinity_container_filters) do
if fltr.name then
fltr.index = nil
arr_add_new(lst,fltr)
end
end
arr_sort(lst,sort_inf_filter)
for i,fltr in pairs(lst) do
fltr.index = i
end
ntt.infinity_container_filters = lst
end
end
local handle_planner_functions = {}
function handle_planner_functions.decon(itm)
local lst = {}
for _,fltr in pairs(itm.entity_filters) do
arr_add_new(lst,fltr)
end
arr_sort(lst,sort_decon_filter)
itm.entity_filters = lst
lst = {}
for _,fltr in pairs(itm.tile_filters) do
arr_add_new(lst,fltr)
end
arr_sort(lst,sort_decon_tile_filter)
itm.tile_filters = lst
end
function handle_planner_functions.upgrade(itm)
local from = {}
local to = {}
local to_only = {}
local fn = itm.get_mapper
for i = 1,itm.mapper_count do
local s = fn(i,"from")
local d = fn(i,"to")
if s and not s.name then s = nil end
if d and not d.name then d = nil end
if s then
to[arr_add_new(from,s)] = d
elseif d then
arr_add_new(to_only,d)
end
end
arr_sort(from,sort_upgrade_filter)
arr_sort(to_only,sort_upgrade_filter)
fn = itm.set_mapper
for i = 1,#from do
fn(i,"from",from[i])
fn(i,"to",to[from[i]])
end
for i = 1,#to_only do
fn(i+#from,"from",nil)
fn(i+#from,"to",to_only[i])
end
for i = 1+#from+#to_only,itm.mapper_count do
fn(i,"from",nil)
fn(i,"to",nil)
end
end
local cfg_defines_inventory_item_main = defines.inventory.item_main
function handle_planner_functions.book(itm)
if itm.object_name ~= "LuaItemStack" then return end
return itm.get_inventory(cfg_defines_inventory_item_main).sort_and_merge()
end
function handle_planner_functions.books(itm,plr)
local cond = {}
for _,v in pairs{"decon","upgrade","book"} do
cond[v] = get_player_settings_value(plr,"sbSortedLogistics-sort-"..v,false)
end
local function recurseR(book)
for _,rec in pairs(book.contents) do
local t = cfg_type_map[rec.type]
local f = cond[t] and handle_planner_functions[t]
if f then f(rec) end
if t == "book" then
recurseR(rec)
end
end
end
local function recurseI(book)
for rec in iter_inventory_slots(book.get_inventory(cfg_defines_inventory_item_main),true) do
local t = cfg_type_map[rec.name]
local f = cond[t] and handle_planner_functions[t]
if f then f(rec) end
if t == "book" then
recurseI(rec)
end
end
end
local tbl = {
["LuaItemStack"] = recurseI,
["LuaRecord"] = recurseR,
}
return tbl[itm.object_name](itm)
end
local function gui_add_spacer(arr,size)
if #arr<1 or arr[#arr].type=="empty-widget" then return end
return arr_insert(arr,{
type="empty-widget",
style_overwrite={
size = size,
},
})
end
local function gui_add_button(plr,arr,name,id)
return arr_insert(arr,{
type="button",
name="sbSortedLogistics_sort"..id,
caption={"sbSortedLogistics-name.btn-sort-"..name},
tooltip={"sbSortedLogistics-desc.btn-sort-"..name},
style_overwrite={
horizontally_stretchable = true,
horizontally_squashable = false,
},
})
end
local function gui_add_setting(plr,arr,name)
return arr_insert(arr,{
type="checkbox",
state=get_player_settings_value(plr,"sbSortedLogistics-sort-"..name,false),
name="sbSortedLogistics_auto_"..name,
caption={"mod-setting-name.sbSortedLogistics-sort-"..name},
tooltip={"mod-setting-description.sbSortedLogistics-sort-"..name},
})
end
local function gui_destroy_deep(el)
for _,e in pairs(el.children) do
local g = e["sbSortedLogistics_gui"]
if g then
g.destroy()
end
gui_destroy_deep(e)
end
end
local function gui_destroy(plr)
local gui = plr.gui.screen["sbSortedLogistics_gui"]
if not gui or get_player_settings_value(plr,"sbSortedLogistics-gui-show","always")=="never" then
return gui_destroy_deep(plr.gui)
end
gui.visible = false
gui.clear()
end
local function gui_find_position(tags,pt_type,default)
if is_not_table(tags) then return default end
tags = tags["sbSortedLogistics"]
if is_not_table(tags) then return default end
tags = tags[pt_type]
if is_not_table(tags) then return default end
return tags
end
local function gui_show(plr,elements,pt_type)
local gui = plr.gui.screen["sbSortedLogistics_gui"]
if not gui then
gui_destroy_deep(plr.gui)
gui = plr.gui.screen.add{
type="frame",
name="sbSortedLogistics_gui",
direction="vertical",
caption={"mod-name.sbSortedLogistics"},
location={0,0},
}
gui.style.horizontally_stretchable = true
gui.style.horizontally_squashable = false
end
gui.clear()
if #elements<1 then
gui.visible = false
return
end
for _,v in pairs(elements) do
local style = v.style_overwrite
if style then v.style_overwrite = nil end
local el = gui.add(v)
if style then
local st = el.style
for k,o in pairs(style) do
st[k] = o
end
end
end
gui.location = gui_find_position(gui.tags,pt_type,{0,0})
gui.visible = true
end
local function gui_moved(el,pt_type)
local dat = el.tags
if is_not_table(dat) then dat = {} end
local db = tbl_get_tbl(dat,"sbSortedLogistics")
db[pt_type] = el.location
el.tags = dat
end
local function gui_show_entity(plr,ntt,typ,when)
data_storage[plr.index] = {ntt,typ}
local elements = {}
if typ then
for k,v in pairs(cfg_inventories[ntt.type] or arr_empty) do
if get_inventory(ntt,v) and (when == "always" or not get_player_settings_value(plr,"sbSortedLogistics-sort-"..typ..k)) then
gui_add_spacer(elements,5)
gui_add_button(plr,elements,typ..k,k)
if when ~= "mini" then
gui_add_setting(plr,elements,typ..k)
end
end
end
end
if get_first_inventory(ntt) and (when == "always" or not get_player_settings_value(plr,"sbSortedLogistics-sort-allinv")) then
gui_add_spacer(elements,5)
gui_add_button(plr,elements,"allinv","a")
if when ~= "mini" then
gui_add_setting(plr,elements,"allinv")
end
end
if has_sortable_logistic_sections(ntt) and (when == "always" or not get_player_settings_value(plr,"sbSortedLogistics-sort-"..(typ or "alllogi"))) then
gui_add_spacer(elements,15)
gui_add_button(plr,elements,typ or "alllogi","l")
if when ~= "mini" then
for _,v in pairs{typ,"alllogi"} do
gui_add_setting(plr,elements,v)
end
end
end
if cfg_has_infinity_filter[ntt.type] and (when == "always" or not get_player_settings_value(plr,"sbSortedLogistics-sort-infinity")) then
gui_add_spacer(elements,15)
gui_add_button(plr,elements,"infinity","i")
if when ~= "mini" then
gui_add_setting(plr,elements,"infinity")
end
end
return gui_show(plr,elements,cfg_special_type[typ] and typ or ntt.name)
end
local function gui_show_planner(plr,bp,typ,when)
data_storage[plr.index] = {bp,typ}
local elements = {}
if when == "always" or not get_player_settings_value(plr,"sbSortedLogistics-sort-"..typ) or (typ=="book" and not get_player_settings_value(plr,"sbSortedLogistics-sort-books")) then
for n,t in pairs{p=typ,b=(typ=="book" and "books" or nil)} do
gui_add_button(plr,elements,t,n)
end
if when ~= "mini" then
for _,t in pairs(typ=="book" and {"decon","upgrade",typ,"books"} or {typ}) do
gui_add_setting(plr,elements,t)
end
end
end
return gui_show(plr,elements,cfg_special_type[typ] and typ or bp.name)
end
local gui_click = {
["sbSortedLogistics_sort1"] = function(_,_,obj)
return handle_entity_inventory(obj,1)
end,
["sbSortedLogistics_sort2"] = function(_,_,obj)
return handle_entity_inventory(obj,2)
end,
["sbSortedLogistics_sorta"] = function(_,_,obj)
return handle_entity_inventory_all(obj)
end,
["sbSortedLogistics_sortl"] = function(_,_,obj)
return handle_entity_logistic_sections(obj)
end,
["sbSortedLogistics_sorti"] = function(_,_,obj)
return handle_entity_infinity_filters(obj)
end,
["sbSortedLogistics_sortp"] = function(_,plr,obj,typ)
return handle_planner_functions[typ](obj,plr)
end,
["sbSortedLogistics_sortb"] = function(_,plr,obj)
return handle_planner_functions["books"](obj,plr)
end,
}
do
local function gui_click_make_auto(id)
local setting_name = "sbSortedLogistics-sort-"..id
gui_click["sbSortedLogistics_auto_"..id] = function(el,plr)
plr.mod_settings[setting_name] = {value=el.state}
end
end
for k,_ in pairs(settings.player_default) do
if str_substr(k,1,23) == "sbSortedLogistics-sort-" then
gui_click_make_auto(str_substr(k,24))
end
end
end
local function evt_handle_entity(plr,open,obj,typ)
if get_player_settings_value(plr,"sbSortedLogistics-sort-allinv") then
handle_entity_inventory_all(obj)
elseif typ then
for k in pairs(cfg_inventories[obj.type] or arr_empty) do
if get_player_settings_value(plr,"sbSortedLogistics-sort-"..typ..k) then
handle_entity_inventory(obj,k)
end
end
end
if get_player_settings_value(plr,"sbSortedLogistics-sort-alllogi") or (typ and get_player_settings_value(plr,"sbSortedLogistics-sort-"..typ)) then
handle_entity_logistic_sections(obj)
end
if cfg_has_infinity_filter[obj.type] and get_player_settings_value(plr,"sbSortedLogistics-sort-infinity") then
handle_entity_infinity_filters(obj)
end
if not open then return end
local when = get_player_settings_value(plr,"sbSortedLogistics-gui-show","always")
if when == "never" then return end
return gui_show_entity(plr,obj,typ,when)
end
local function evt_handle_planner(plr,open,obj,typ)
if get_player_settings_value(plr,"sbSortedLogistics-sort-"..typ) then
handle_planner_functions[typ](obj,plr)
end
if typ=="book" and get_player_settings_value(plr,"sbSortedLogistics-sort-books") then
handle_planner_functions["books"](obj,plr)
end
if not open then return end
local when = get_player_settings_value(plr,"sbSortedLogistics-gui-show","always")
if when == "never" then return end
return gui_show_planner(plr,obj,typ,when)
end
local cfg_defines_controllers_character = defines.controllers.character
local evt_gui_open_close_func = {
[defines.gui_type.controller]=function(_,plr,open)
local chr = plr.character
if chr and plr.controller_type == cfg_defines_controllers_character then
return evt_handle_entity(plr,open,chr,"self")
end
end,
[defines.gui_type.entity]=function(ev,plr,open)
local ntt = ev.entity
return evt_handle_entity(plr,open,ntt,cfg_type_map[ntt.type])
end,
[defines.gui_type.item]=function(ev,plr,open)
local bp = ev.item
local typ = cfg_type_map[bp.name]
if typ then
return evt_handle_planner(plr,open,bp,typ)
end
end,
}
script.on_event(defines.events.on_gui_location_changed, safecall_wrapper(function(ev)
local el = ev.element
local obj,typ = arr_unpack(data_storage[ev.player_index] or arr_empty)
if el.name=="sbSortedLogistics_gui" and obj and obj.valid then
return gui_moved(el,cfg_special_type[typ] and typ or obj.type)
end
end))
script.on_event(defines.events.on_gui_click, safecall_wrapper(function(pEvent)
local fn = gui_click[pEvent.element.name]
local obj,typ = arr_unpack(data_storage[pEvent.player_index] or arr_empty)
if fn and typ and obj and obj.valid then
return fn(pEvent.element,get_player(pEvent.player_index),obj,typ)
end
end))
script.on_event(defines.events.on_gui_opened, safecall_wrapper(function(pEvent)
local plr = get_player(pEvent.player_index)
gui_destroy(plr)
data_storage[plr.index] = nil
local fn = evt_gui_open_close_func[pEvent.gui_type]
if fn then return fn(pEvent,plr,true) end
end))
script.on_event(defines.events.on_gui_closed, safecall_wrapper(function(pEvent)
local plr = get_player(pEvent.player_index)
gui_destroy(plr)
data_storage[plr.index] = nil
local fn = evt_gui_open_close_func[pEvent.gui_type]
if fn then fn(pEvent,plr,false) end
fn = plr.character
if plr.opened_self and fn and plr.controller_type == cfg_defines_controllers_character then return evt_handle_entity(plr,true,fn,"self") end
fn = plr.opened
if fn and fn.object_name == "LuaEntity" then return evt_handle_entity(plr,true,fn,cfg_type_map[fn.type]) end
end))
script.on_event(defines.events.on_player_cursor_stack_changed, safecall_wrapper(function(pEvent)
local plr = get_player(pEvent.player_index)
local typ
local itm = plr.cursor_record
if itm and itm.valid_for_write then
typ = itm.type
else
itm = plr.cursor_stack
if itm and itm.valid_for_read then
typ = itm.name
end
end
typ = cfg_type_map[typ]
if typ then
return evt_handle_planner(plr,nil,itm,typ)
end
end))
local function get_inventories(ntt)
local ret = {}
local get = ntt.get_inventory
for n = 1,ntt.get_max_inventory_index() do
local i = get(n)
if i and #i>0 then
ret[n] = i
end
end
return ret
end
game.player.get_main_inventory().find_item_stack{name="scrap"}
game.player.opened.get_inventory(4)[4]
/c
local t = game.player.opened.get_inventory(3)
local i = game.player.opened.get_inventory(3)[2]
log(serpent.block({
t.find_empty_stack(i).transfer_stack(i)
},{comment=false,sortkeys=false,sparse=true,compact=true,indent="\t"}))
find_empty_stack(item
/c
local o = game.player.opened
local t = {}
for n=1,o.get_max_inventory_index(),1 do
local i = o.get_inventory(n)
if i then t[n] = i.get_contents() end
end
log(serpent.block(t,{comment=false,sortkeys=false,sparse=true,compact=true,indent="\t"}))
find_empty_stack(item
/c
log(serpent.block({
defines["prototypes"]
},{comment=false,sortkeys=false,sparse=true,compact=true,indent="\t"}))